class AoipyErrorHandling:

    def Errors(self, error, code):
        if error == 1:
            raise SyntaxError(f"ID '{code}' is not a user ID")
        if error == 2:
            raise SyntaxError(f"ID '{code}' is not a channel ID")

        if error == 4:
            raise SyntaxError(code)



import Aoipy.AoiErrorHandling as ErrorHandling

EH = ErrorHandling.AoipyErrorHandling()


async def currentChannelID(empty, Context):
    try:
        int(Context.channel.id)
    except ValueError:
        EH.Errors(1, "None")
    return Context.channel.id



import Aoipy.AoiErrorHandling as ErrorHandling

EH = ErrorHandling.AoipyErrorHandling()


async def channelMention(ID, Context):
    if len(ID) < 1:
        raise EH.Errors(4, "No parameter provided for '$channelMention'")
    from Aoipy.Functions.AoiCore import bots
    try:
        int(ID)
        channel = await bots.fetch_channel(ID)
        return channel.mention
    except ValueError:
        EH.Errors(2, ID)


from Aoipy.tools import findBracketPairs

########################################
#              COMMANDS                #
########################################


class Commands:
    # Global variables

    def command(self, Name, Code):
        # Define command function dynamically
        from Aoipy.Classes.AoiPyClient import bots
        @bots.command(name=Name)
        async def go(ctx, *args, Code=Code):
            from Aoipy.Classes.AoiPyClient import TotalFuncs
            Context = ctx
            if '$args' in Code:
                while "$args" in str(Code):
                    count = 0
                    end = None
                    balance = 0
                    start = Code.index("$args") + 5
                    look = Code[start:len(Code)]
                    for i in look:
                        if i == "[":
                            start = count
                            count += 1
                            balance += 1
                            continue
                        if i == "]":
                            end = count
                            balance -= 1
                        count += 1
                        if balance == 0 and start is not None and end is not None:
                            try:
                                # Replace $args with arguments
                                Code = str(Code).replace(f"$args[{look[start + 1:end]}]", args[int(look[start + 1:end]) - 1])
                                break
                            except IndexError:
                                raise SyntaxError(F"$args[{int(look[start + 1:end])}] Not Provided")

            if "$argCheck" in Code:
                if Code.count("$argCheck") > 1:
                    raise Exception("Too many $argCheck in a single command | Max is 1!")
                start = Code.index("$argCheck[") + 10
                area = Code[start:]
                try:
                    argTotal = area[:area.index(";")]
                    warning = area[area.index(";") + 1:area.index("]")]
                    if len(args) != int(argTotal):
                        await Context.channel.send(warning)
                        return
                    Code = Code.replace(f"$argCheck[{argTotal}{area[area.index(';'):area.index(']')]}]\n", "")
                except:
                    raise SyntaxError("Not enough arguments in $argCheck!")

            await findBracketPairs(Code, TotalFuncs, Context)


import Aoipy.AoiErrorHandling as ErrorHandling

EH = ErrorHandling.AoipyErrorHandling()


async def authorID(emp, Context):
    try:
        int(Context.author.id)
    except ValueError:
        EH.Errors(1, "None")
    return Context.author.id



async def sendMessage(args: str, Context):
    from Aoipy.Functions.AoiCore import bots
    split = args.split(";")
    channel_id = split[0]
    message = split[1]
    channel = await bots.fetch_channel(int(channel_id))
    await channel.send(message)
    return


import discord
import random
from discord.ext import tasks


def setActivity(message: str, type: str = "watching"):
    if type.lower() == "watching":
        act = discord.Activity(type=discord.ActivityType.watching, name=message)
    elif type.lower() == "playing":
        act = discord.Activity(type=discord.ActivityType.playing, name=message)
    else:
        act = discord.Activity(type=discord.ActivityType.listening, name=message)

    return act

#Come back to this
@tasks.loop(seconds=10)
async def updateActivity(messages: list, type: str = "watching"):
    from Aoipy.Functions.AoiCore import bots
    all = messages
    await bots.change_presence(activity=discord.Activity(type=discord.ActivityType.watching, name=random.choice(all)))



import discord
from discord.ext import commands
from Aoipy.tools import FunctionHandler

global TotalFuncs
global bots


class AoipyClient(discord.Client):
    # Global variables
    global bots

    def __init__(self, prefix: str, case_insensitive: bool = False, intents: tuple = ("default",), activity=None, help_command=None):
        # Global variables
        global bots

        # Initialize Start class
        self.prefix = prefix
        self.case_insensitive = case_insensitive
        self.intented = intents
        self._help_command = help_command
        self._clients = ''
        self.intent = ''
        self._activity = activity

        # Determine intents
        if "all" in self.intented:
            self.intent = discord.Intents.all()
        elif "default" in self.intented:
            self.intent = discord.Intents.default()
        else:
            self.intent = discord.Intents.default()

        # Enable specific intents
        if "message" in self.intented:
            self.intent.message_content = True
        if "members" in self.intented:
            self.intent.members = True
        if "presences" in self.intented:
            self.intent.presences = True

        # Create bot instances
        if self._activity is None:
            self._clients = commands.Bot(command_prefix=self.prefix, case_insensitive=self.case_insensitive, intents=self.intent,
                                         help_command=self._help_command)
            bots = self._clients

        else:
            self._clients = commands.Bot(command_prefix=self.prefix, case_insensitive=self.case_insensitive, intents=self.intent,
                                         activity=self._activity,
                                         help_command=self._help_command)
            bots = self._clients
        super().__init__(intents=self.intent)

    @property
    def clients(self):
        return self._clients


def Bot(prefix: str, case_insensitive: bool = False, intents: tuple = ("default",), activity=None, help_command=None):
    # Global variables
    global bots
    global TotalFuncs

    # Initialize FunctionHandler and register functions
    Functions = FunctionHandler()
    TotalFuncs = Functions
    Functions.register_functions()

    # Create Start instance and return working bot
    _final = AoipyClient(prefix, case_insensitive, intents, activity, help_command)
    working = _final.clients
    return working



import Aoipy.AoiErrorHandling as ErrorHandling

EH = ErrorHandling.AoipyErrorHandling()


async def username(user):
    from Aoipy.Functions.AoiCore import bots
    try:
        int(user)
        new_user = await bots.fetch_user(user)
    except ValueError:
        EH.Errors(1, user)
        return
    return new_user



with open("Aoipy/all_Functions.txt") as funcs:
    alls = funcs.readlines()
    for i in alls:
        try:
            exec(f'from .{i.replace("$", "").strip()} import *')
        except ModuleNotFoundError:
            continue


from Aoipy.tools import findBracketPairs, FunctionHandler
from Aoipy.Functions import *
import os

# Global variables
global bots




    ########################################
    #              EVENTS                  #
    ########################################


class AoiEvents:
    def onReady(self, code):
        # Define on_ready event function
        @bots.event
        async def on_ready():
            await findBracketPairs(code, TotalFuncs, None)




from Aoipy.Functions import *

class FunctionHandler:

    def __init__(self):
        self.funcs = {}

    def register_functions(self):
        with open("Aoipy/all_Functions.txt", "r") as keys:
            keyword = keys.readlines()
            for line in keyword:
                function = eval(line.replace("$", ""))
                self.funcs[line.replace("\n", "").lower()] = function

    async def execute_functions(self, keyword, args, context):
        return await self.funcs[keyword](args, context)


async def findBracketPairs(entry: str, Functions, context):
    lines = [line.strip() for line in entry.split("\n") if len(line.strip()) >= 3]
    for code in lines:
        first = None
        last = None
        count = 0
        balance1 = 0
        for i in code:
            if i == "[" and first is None:
                first = count
                balance1 += 1
                count += 1
                continue
            if i == "[":
                balance1 += 1
            elif i == "]":
                last = count
                balance1 -= 1
            if first is not None and last is not None and balance1 == 0:
                break
            count += 1
        argument = str(code[first + 1:last])
        keyword = code[0:first]
        find = [first, last, keyword, argument, context]
        while "[" in str(argument) and "]" in str(argument) and "$" in str(argument):
            count = 0
            start = None
            end = None
            balance = 0
            for i in argument:
                if i == "$" and start is None:
                    start = count
                elif i == "[":
                    balance += 1
                elif i == "]":
                    end = count
                    balance -= 1
                count += 1
                if balance == 0 and start is not None and end is not None:
                    break
            if start != 0:
                argument = argument[:start] + str(await findBracketPairs(argument[start:end + 1], Functions, context)) + argument[end + 1:]
            else:
                argument = str(await findBracketPairs(argument, Functions, context)) + argument[end + 1:]
            find = [first, last, keyword, argument, context]
        if find[2].lower() in Functions.funcs:
            name = await Functions.execute_functions(find[2].lower(), find[3], find[4])
        else:
            name = find[2]

    try:
        return name
    except Exception as e:
        raise Exception(f"Error at: {e}")



from Aoipy.tools import findBracketPairs

#######################################
#              EVENTS                 #
#######################################


class AoiEvents:
    def onReady(self, code):
        from Aoipy.Classes.AoiPyClient import bots, TotalFuncs
        # Define on_ready event function
        @bots.event
        async def on_ready():
            await findBracketPairs(code, TotalFuncs, None)


import setuptools
with open("readMe.md", "r") as rd:
    longtext = rd.read()

setuptools.setup(
    # Here is the module name.
    name="AoiPy",
    version="0.11.1Beta",
    author="Jade",
    description="Aoi.py is the best python string-based package for Discord bot devs",
    long_description=longtext,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),

    # if module has dependencies i.e. if your package rely on other package at pypi.org
    # then you must add there, in order to download every requirement of package
    install_requires=["Py-cord", "Discord"],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)